/*
 * General functions
 */

// tab initialization
TabMix._initTab = function (aTab) {
  aTab.groupID = null;
  aTab._opener = null;
};

// clear opener from all tabs
TabMix._clearOpener = function () {
  Array.forEach( gBrowser.visibleTabs, function (tab) {
  if ( !tab.groupID )
    tab._opener = null;
  });
};

// Get the next or the previous visible tab
//   We can't use gBrowser.visibleTabs here, because aTab is removed from the array 
//   in the case of closing a tab (when gBrowser.removeTab() is called)
//
TabMix.nextTab = function (aTab) {
  let next = null;
  let tab = aTab;
  while ( tab.nextSibling ) {
    tab = tab.nextSibling;
    if ( !tab.hidden ) {
      next = tab;
      break;
    }
  }
  return next;
};

TabMix.prevTab = function (aTab) {
  let prev = null;
  let tab = aTab;
  while ( tab.previousSibling ) {
    tab = tab.previousSibling;
    if ( !tab.hidden ) {
      prev = tab;
      break;
    }
  }
  return prev;
};

/*
 * Tab features
 */

// Move Tab
//   this function is used to move tab without passing onTabMove function
TabMix.moveTabTo = function TM_moveTabTo( aTab, aPos ) {
  gBrowser.moveTabTo( aTab, aPos );
};

// Protect tab
//   Automatically lock a protected tab
TabMix.protectTab = function (aTab) {
  if ( aTab.hasAttribute("protected") ) {
    aTab.removeAttribute("protected");
    aTab.removeAttribute("locked");
  }
  else {
    aTab.setAttribute("protected", "true");
    if ( __TM_Prefs.getBoolPref("lockProtectedTab") )
      aTab.setAttribute("locked", "true");
  }
};

// Pin or Unpin tab
TabMix.pinOrUnpinTab = function (aTab) {
  if ( aTab.hasAttribute("pinned") && aTab.getAttribute("pinned") == "true" )
    gBrowser.unpinTab(aTab);
  else
    gBrowser.pinTab(aTab);
};

// Duplicate Tab
//   Use gBrowser.duplicateTab() to duplicate Tab.
//   Then call gBrowser.moveToTab() to move the new tab next to the original one.
//
TabMix.duplicateTab = function (aTab) {
  var newTab = gBrowser.duplicateTab(aTab);
  gBrowser.moveTabTo(newTab, aTab._tPos+1);
  TabMixGroup.addFromTab(newTab, aTab);
  newTab._opener = aTab._opener;
};  
  
// Close Right Tabs and Left Tabs
//   aDir =  1 : right tabs
//        = -1 : left tabs
//  
TabMix.closeMultipleTabs = function (aDir) {
  let tabs = gBrowser.visibleTabs;
  let tab = TabContextMenu.contextTab;
  let index = tabs.indexOf(tab);
  
  let next = 1;
  let prev = -1;
  let ltr = (window.getComputedStyle(gBrowser.tabContainer).direction == "ltr");
  if (!ltr)
    aDir *= -1;
    
  let start = ( aDir == prev ) ? index - 1 : tabs.length - 1;
  let stop  = ( aDir == prev ) ? gBrowser._numPinnedTabs : index + 1;
  let tabsToClose = start - stop + 1;
  
  if ( tabsToClose < 1 || !gBrowser.TMwarnAboutClosingTabs( false, tabsToClose ) )
    return;

  for ( let i = start; i >= stop; i-- )
    gBrowser.removeTab(tabs[i]);
};

/*
 * Colorful Tab Grouping functions
 *   These are the functions accesible to user
 */

TabMix.toggleFolder = function (aTab) {
  let ID = aTab.groupID;
  if ( !ID )
    return;

  if ( aTab.hasAttribute("hidden") )
    TabMixGroup.openStack(ID);
  else
    TabMixGroup.makeStack(aTab);
};

TabMix.bookmarkGroup = function (aTab) {
  let tabs = TabMixGroup.getTabsInGroup(aTab);
  let URIs = tabs.map( function __getUri(tab) {
               return tab.linkedBrowser.webNavigation.currentURI;
             });
  PlacesUIUtils.showMinimalAddMultiBookmarkUI(URIs);
};

TabMix.reloadGroup = function (aTab) {
  let tabs = TabMixGroup.getTabsInGroup(aTab);
  for ( var i = 0; i < tabs.length; i++ ) {
    try {
      gBrowser.getBrowserForTab(tabs[i]).reload();
    }
    catch (e) {}
  }
};

TabMix.groupOrphanTabs = function (aTab) {
  let tabs = gBrowser.visibleTabs;
  let tab = aTab.nextSibling;
  while ( tab && !tab.hasAttribute("groupID") ) {
    if ( tabs.indexOf(tab) >= 0 && !tab.pinned ) // make sure |tab| is in the current Tabview group
      TabMixGroup.addFromTab( tab, aTab );
    tab = tab.nextSibling;
  }
  tab = aTab.previousSibling;
  while ( tab && !tab.hasAttribute("groupID") ) {
    if ( tabs.indexOf(tab) >= 0 && !tab.pinned )
      TabMixGroup.addFromTab( tab, aTab );
    tab = tab.previousSibling;
  }
};

// Remove one tab from its group
//
TabMix.removeFromGroup = function TM_removeFromGroup (aTab) {
  let ID = aTab.groupID;
  if ( ID == null )
    return;
  TabMixGroup.removeOneTab(aTab);
  
  // Then we need to move aTab to the end of the group
  let tabs = TabMixGroup.getTabsInGroup(ID);
  if ( tabs.length > 0 && aTab._tPos > tabs[0]._tPos && aTab._tPos < tabs[tabs.length - 1]._tPos )
    this.moveTabTo( aTab, tabs[tabs.length - 1]._tPos );
};

TabMix.undoGroup = function (aTab) {
  let tabs = TabMixGroup.getTabsInGroup(aTab);
  
  // the tabs[0] will automatically removed when we remove the last one, see TabMixGroup.removeOneTab()
  for ( let i = 1; i < tabs.length; i++ ) { 
    TabMixGroup.removeOneTab(tabs[i]);
    tabs[i]._opener = null;
  }
  tabs[0]._opener = null;
};

TabMix.closeAllTabsInGroup = function (aTab) {
  let tabs = TabMixGroup.getTabsInGroup(aTab);
  let last = null;
  
  let tabToClose = tabs.length;
  if ( gBrowser.TMwarnAboutClosingTabs(false, tabToClose) ) {
    for ( let i = tabs.length - 1; i >= 0; i-- ) {
      // if the current tab is in the group, we remove it last
      if ( tabs[i] != gBrowser.mCurrentTab )
        gBrowser.removeTab(tabs[i]);
      else
        last = tabs[i];
    }
    if ( last )
      gBrowser.removeTab(last);
  }
};

